-- -----------------------------------------------------------------------------
-- -----------------------------------------------------------------------------
--  Hammer of Justas
--
-- Copyright (C) 2010-2020
-- By Jonathan E. Wright
--
-- FILE: HoJ_Log.lua
-- DESC: Addon logging functions.

local addonName, HoJ = ...

-- DEFAULT_CHAT_FRAME:AddMessage( "HoJ_Log.lua" )

-- ============================================================================

-- -----------------------------------------------------------------------------
-- ToStr
-- -----------------------------------------------------------------------------
function ToStr( value )
	if ( value == nil ) then
		return "nil"
	end
	return tostring( value )
end

-- -----------------------------------------------------------------------------
-- TableMemberToStr
-- -----------------------------------------------------------------------------
function TableMemberToStr( tbl, member )
	if ( tbl == nil ) then
		return "nil table"
	elseif ( tbl[member] == nil ) then
		return "nil"
	end
	return tostring( tbl[member] )
end

-- -----------------------------------------------------------------------------
-- HoJ:SplitString
-- -----------------------------------------------------------------------------
function HoJ:SplitString( inStr, separator )
	if type( inStr ) ~= "string" then
		HoJ:Error( "parsed parameter is not a table." )
		return
	end

	if ( separator == nil ) then
		separator = "%s"
	end

	local out = { }
	for str in string.gmatch( inStr, "([^" .. separator .. "]+)" ) do
		table.insert( out, str )
	end
	return out
end

-- -----------------------------------------------------------------------------
-- HoJ:CinematicInProgress
-- returns true if InCinematic() == true OR is we received a CINEMATIC_START
-- event and haven't received a corresponding CINEMATIC_STOP event yet.
-- -----------------------------------------------------------------------------
function HoJ:CinematicInProgress()
	return HoJ._inCinematic or InCinematic() or IsInCinematicScene()
end

-- -----------------------------------------------------------------------------
-- HoJ:GetVersionString
-- -----------------------------------------------------------------------------
function HoJ:GetVersionString()
	local version = "" .. HoJ.MAJOR_VERSION .. "." .. HoJ.MINOR_VERSION .. "." .. HoJ.BUILD_VERSION .. "-" .. HoJ.BUILD_TYPE
	return version
end


-- -----------------------------------------------------------------------------
-- HoJ:Error
-- -----------------------------------------------------------------------------
function HoJ:Error( text )
	if ( DEFAULT_CHAT_FRAME ) then
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_RED .. "HoJ." .. HoJ.STR_WHITE .. text )
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:ErrorIf
-- -----------------------------------------------------------------------------
function HoJ:ErrorIf( test, text )
	if ( test == false ) then
		return
	end

	if ( DEFAULT_CHAT_FRAME ) then
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_RED .. "HoJ." .. HoJ.STR_WHITE .. text )
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:FTError
-- -----------------------------------------------------------------------------
function HoJ:FTError( numFrames, text )
	if ( numFrames == nil or type( numFrames ) ~= "number" ) then
		HoJ:Error( "FTError passed invalid parameters!" )
		HoJ:DebugBreak()
	end

	if ( ( HoJ._frameNum % numFrames ) ~= 0 ) then
		return
	end
	HoJ:Error( text )
end

-- -----------------------------------------------------------------------------
-- HoJ:ErrorOnce
-- -----------------------------------------------------------------------------
function HoJ:ErrorOnce( text )
	if ( HoJ._errorLogs == nil ) then
		HoJ._errorLogs = { }
	else
		if ( HoJ._errorLogs[text] ~= nil ) then 
			return
		end
	end
	HoJ._errorLogs[text] = true
	HoJ:Error( text )
end

-- -----------------------------------------------------------------------------
-- HoJ:Log
-- -----------------------------------------------------------------------------
function HoJ:Log( text )
	if ( DEFAULT_CHAT_FRAME ) then
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_YELLOW .. "HoJ(" .. HoJ.STR_GREEN .. string.format( "%.2f", HoJ._curTime ) .. HoJ.STR_YELLOW .. "): " .. HoJ.STR_WHITE .. text )
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:LogOnce
-- -----------------------------------------------------------------------------
function HoJ:LogOnce( text )
	if ( HoJ._logs == nil ) then
		HoJ._logs = { }
	else
		if ( HoJ._logs[text] ~= nil ) then 
			return
		end
	end
	HoJ._logs[text] = true
	HoJ:Log( text )
end

-- -----------------------------------------------------------------------------
-- HoJ:LogD
-- -----------------------------------------------------------------------------
function HoJ:LogD( show, text )
	if ( show and DEFAULT_CHAT_FRAME ) then
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_YELLOW .. "HoJ(" .. HoJ.STR_GREEN .. string.format( "%.2f", HoJ._curTime ) .. HoJ.STR_YELLOW .. "): " .. HoJ.STR_WHITE .. text )
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:FTLog
-- -----------------------------------------------------------------------------
function HoJ:FTLog( numFrames, text )
	if ( numFrames == nil or type( numFrames ) ~= "number" ) then
		HoJ:Error( "FTLog passed invalid parameters!" )
		HoJ:DebugBreak()
	end

	if ( ( HoJ._frameNum % numFrames ) ~= 0 ) then
		return
	end
	HoJ:Log( text )
end

-- -----------------------------------------------------------------------------
-- HoJ:FTLogD
-- -----------------------------------------------------------------------------
function HoJ:FTLogD( numFrames, show, text )
	if ( numFrames == nil or type( numFrames ) ~= "number" ) then
		HoJ:Error( "FTLogD passed invalid parameters!" )
		HoJ:DebugBreak()
	end

	if ( ( HoJ._frameNum % numFrames ) ~= 0 ) then
		return
	end
	HoJ:LogD( show, text )
end

-- -----------------------------------------------------------------------------
-- HoJ:DebugMsg
-- -----------------------------------------------------------------------------
function HoJ:DebugMsg( text )
	if ( DEFAULT_CHAT_FRAME ) then
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_YELLOW .. "HoJ(" .. HoJ.STR_GREEN .. string.format( "%.2f", HoJ._curTime ) .. HoJ.STR_YELLOW .. "): " .. HoJ.STR_WHITE .. text )
		-- to enable frame number spam
		--DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_PURPLE .. "(" ..HoJ._frameNum .. "):" .. HoJ.STR_YELLOW .. "HoJ." .. HoJ.STR_WHITE .. text );
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:ChannelInfo
-- -----------------------------------------------------------------------------
function HoJ:ChannelInfo( text )
--[[
	if ( DEFAULT_CHAT_FRAME ) then
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_GREEN .. "HoJ." .. HoJ.STR_WHITE .. text )
	end
--]]
end

-- -----------------------------------------------------------------------------
-- HoJ:ChannelError
-- -----------------------------------------------------------------------------
function HoJ:ChannelError( text )
	if ( DEFAULT_CHAT_FRAME ) then
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_RED .. "HoJ." .. HoJ.STR_WHITE .. text )
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:ThrottleMsg
-- -----------------------------------------------------------------------------
function HoJ:ThrottleMsg( interval, text )
	if ( DEFAULT_CHAT_FRAME ) then
--[[
		local msg = HoJ._throttledMessages[text]
		if ( msg == nil ) then
			msg = {
				lastTime = self._curTime
			}
			HoJ._throttledMessages[text] = msg
		else
			local timeSincePrinted = self._curTime - msg.lastTime
			if ( timeSincePrinted < interval ) then
				return
			end
		end

		msg.lastTime = self._curTime
--]]
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_YELLOW .. "HoJ(" .. HoJ.STR_GREEN .. string.format( "%.2f", HoJ._curTime ) .. HoJ.STR_YELLOW .. "): " .. HoJ.STR_WHITE .. text )
		-- to enable frame number spam
		--DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_PURPLE .. "(" ..HoJ._frameNum .. "):" .. HoJ.STR_YELLOW .. "HoJ." .. HoJ.STR_WHITE .. text );
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:FrameThrottleMsg
-- -----------------------------------------------------------------------------
function HoJ:FrameThrottleMsg( numFrames, text )
	if ( DEFAULT_CHAT_FRAME ) then
--[[
		local msg = HoJ._throttledMessages[text]
		if ( msg == nil ) then
			msg = {
				lastTime = self._curTime
			}
			HoJ._throttledMessages[text] = msg
		else
			local timeSincePrinted = self._curTime - msg.lastTime
			if ( timeSincePrinted < interval ) then
				return
			end
		end

		msg.lastTime = self._curTime
--]]
		if ( ( HoJ._frameNum % numFrames ) ~= 0 ) then
			return
		end
		-- without frame number spam
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_YELLOW .. "HoJ(" .. HoJ.STR_GREEN .. string.format( "%.2f", HoJ._curTime ) .. HoJ.STR_YELLOW .. "): " .. HoJ.STR_WHITE .. text )
		-- to enable frame number spam
		--DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_PURPLE .. "(" ..HoJ._frameNum .. "):" .. HoJ.STR_YELLOW .. "HoJ." .. HoJ.STR_WHITE .. text );
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:Warn
-- -----------------------------------------------------------------------------
function HoJ:Warn( text )
	-- should be commented out for releases
	if ( DEFAULT_CHAT_FRAME ) then
		--DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_PURPLE .. "(" ..HoJ._frameNum .. "):" .. HoJ.STR_YELLOW .. "HoJ." .. HoJ.STR_WHITE .. text );
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.STR_RED .. "HoJ." .. HoJ.STR_WHITE .. text );
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:Print
-- -----------------------------------------------------------------------------
function HoJ:Print( text )
	if ( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage( text )
	end
end

-- -----------------------------------------------------------------------------
-- HoJ:Msg
-- -----------------------------------------------------------------------------
function HoJ:Msg( text )
	if ( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage( HoJ.MSG_PREFIX .. text )
	end
end
